<?php


class wp_slimstat_custom_db{
	public static $slug = 'wp-slimstat-custom-db';

	/**
	 * Initializes the environment
	 */
	public static function init(){
		if (!class_exists('wp_slimstat')) return true;

		// Update checker
		if (!empty(wp_slimstat::$options['addon_licenses'][self::$slug])){
			add_action('wp_loaded', array(__CLASS__, 'update_checker'));
		}

		// Pass the custom wpdb object to WP SlimStat
		add_filter('slimstat_custom_wpdb', array(__CLASS__, 'custom_wpdb'));

		// Adds the options to specify database name, username, password and host
		add_filter('slimstat_init_options', array(__CLASS__, 'init_options'));

		// Adds the options to specify database name, username, password and host
		add_filter('slimstat_options_on_page', array(__CLASS__, 'add_db_params_to_options_page'));

		// Load localization files
		load_plugin_textdomain(self::$slug, false, self::$slug . '/lang/');
	}

	/**
	 * Checks for updates
	 */
	public static function update_checker(){
		@include_once(plugin_dir_path( __FILE__ ).'update-checker.php');
		$MyUpdateChecker = new PluginUpdateChecker('http://slimstat.getused.to.it/update-checker/?slug='.self::$slug.'&key='.urlencode(wp_slimstat::$options['addon_licenses'][self::$slug]), __FILE__, self::$slug);
	}

	public static function custom_wpdb($_wpdb = ''){
		$new_wpdb = new wpdb(wp_slimstat::$options['addon_custom_db_dbuser'], wp_slimstat::$options['addon_custom_db_dbpass'], wp_slimstat::$options['addon_custom_db_dbname'], wp_slimstat::$options['addon_custom_db_dbhost']);
		$new_wpdb->hide_errors();

		$counter = 0;

		// Check if the slim_stats table is there. If found, assume that all the other tables are in place as well, otherwise init all tables
		foreach ($new_wpdb->get_col("SHOW TABLES LIKE '{$GLOBALS['wpdb']->prefix}slim\_stats%'", 0) as $a_table) $counter++;

		if ($counter == 0){
			include_once(WP_PLUGIN_DIR.'/wp-slimstat/admin/wp-slimstat-admin.php');
			wp_slimstat_admin::init_tables($new_wpdb);
		}

		return $new_wpdb;
	}

	/**
	 * Adds a new option to the Advanced Settings tab
	 */
	public static function add_db_params_to_options_page($_options = array()){
		$_options['addon_custom_db_dbhost'] = array('description' => __('External DB Host',self::$slug), 'type' => 'text', 'long_description' => __('Name of the host containing the external database where you want to keep all the information about your visitors',self::$slug));
		$_options['addon_custom_db_dbname'] = array('description' => __('External DB Name',self::$slug), 'type' => 'text', 'long_description' => __('Name of the external database',self::$slug));
		$_options['addon_custom_db_dbuser'] = array('description' => __('External DB User',self::$slug), 'type' => 'text', 'long_description' => __('Username associated to the database',self::$slug));
		$_options['addon_custom_db_dbpass'] = array('description' => __('External DB Pass',self::$slug), 'type' => 'text', 'long_description' => __('Password to let Keyword Swarm access and update the data',self::$slug));
		return $_options;
	}

	/**
	 * Modifies the value associated to the ip_lookup_url option
	 */
	public static function init_options($_options){
		$_options['addon_custom_db_dbhost'] = !isset($_options['addon_custom_db_dbhost'])?DB_HOST:$_options['addon_custom_db_dbhost'];
		$_options['addon_custom_db_dbname'] = !isset($_options['addon_custom_db_dbname'])?DB_NAME:$_options['addon_custom_db_dbname'];
		$_options['addon_custom_db_dbuser'] = !isset($_options['addon_custom_db_dbuser'])?DB_USER:$_options['addon_custom_db_dbuser'];
		$_options['addon_custom_db_dbpass'] = !isset($_options['addon_custom_db_dbpass'])?DB_PASSWORD:$_options['addon_custom_db_dbpass'];
		return $_options;
	}
}
// end of class declaration

// Bootstrap
if (function_exists('add_action')){
	add_action('plugins_loaded', array('wp_slimstat_custom_db', 'init'), 8);
}