<?php


class wp_slimstat_default_filters{
	public static $slug = 'wp-slimstat-default-filters';

	/**
	 * Initializes the environment
	 */
	public static function init(){
		if (!class_exists('wp_slimstat')) return true;

		// Update checker
		add_action('wp_loaded', array(__CLASS__, 'update_checker'));

		// Sets the default filters
		add_filter('slimstat_db_filters_normalized', array(__CLASS__, 'set_timeframe'), 10, 2);
		
		// Add a new option to the Add-ons tab
		add_filter('slimstat_options_on_page', array(__CLASS__, 'add_filter_string'));
	}
	
	/**
	 * Checks for updates
	 */
	public static function update_checker(){
		if (!empty(wp_slimstat::$options['addon_licenses'][self::$slug])){
			@include_once(plugin_dir_path( __FILE__ ).'update-checker.php');
			$MyUpdateChecker = new PluginUpdateChecker('http://slimstat.getused.to.it/update-checker/?slug='.self::$slug.'&key='.urlencode(wp_slimstat::$options['addon_licenses'][self::$slug]), __FILE__, self::$slug);
		}
	}

	/**
	 * Sets the default filters
	 */
	public static function set_timeframe($_filters_normalized = array(), $_filters = ''){
		if (empty(wp_slimstat::$options['addon_default_filters_string'])) return $_filters_normalized;

		$default_filters = wp_slimstat_db::parse_filters(wp_slimstat::$options['addon_default_filters_string']);
		
		foreach($default_filters as $a_group_key => $a_group_value){
			foreach($a_group_value as $a_key => $a_value){
				if (empty($_filters_normalized[$a_group_key][$a_key])){
					$_filters_normalized[$a_group_key][$a_key] = $a_value;
				}
			}
		}

		return $_filters_normalized;
	}
	
	/**
	 * Adds a new option to the Advanced Settings tab
	 */
	public static function add_filter_string($_options = array()){
		$_options['addon_default_filters_header'] = array('description' => __('Default Filters',self::$slug), 'type' => 'section_header');
		$_options['addon_default_filters_string'] = array('description' => __('Filter String',self::$slug), 'type' => 'text', 'long_description' => __('Please refer to <a href="#">these FAQs</a> for more information on how to define filters. They look like <code>year equals this year&&&browser contains fire</code>.',self::$slug));
		return $_options;
	}
}
// end of class declaration

// Bootstrap
if (function_exists('add_action')){
	add_action('plugins_loaded', array('wp_slimstat_default_filters', 'init'), 15);
}