<?php


class wp_slimstat_export_to_excel{
	public static $slug = 'wp-slimstat-export-to-excel';

	/**
	 * Initializes the environment
	 */
	public static function init(){
		if (!class_exists('wp_slimstat')) return true;

		// Update checker
		if (!empty(wp_slimstat::$options['addon_licenses'][self::$slug])){
			add_action('wp_loaded', array(__CLASS__, 'update_checker'));
		}

		// Adds a new 'save' icon to the reports
		add_filter('slimstat_report_header_buttons', array(__CLASS__, 'add_save_icon'), 10, 2);
		
		// Defines what happens when the user clicks the save button 
		add_action('init', array(__CLASS__, 'generate_xls'));

		// Get all the results
		add_filter('slimstat_post_filters_normalized', array(__CLASS__, 'no_limits'));

		// Localization files
		load_plugin_textdomain(self::$slug, false, self::$slug . '/lang/');
	}

	/**
	 * Checks for updates
	 */
	public static function update_checker(){
		@include_once(plugin_dir_path( __FILE__ ).'update-checker.php');
		$MyUpdateChecker = new PluginUpdateChecker('http://slimstat.getused.to.it/update-checker/?slug='.self::$slug.'&key='.urlencode(wp_slimstat::$options['addon_licenses'][self::$slug]), __FILE__, self::$slug);
	}

	/**
	 * Adds a new 'save' icon to the reports
	 */
	public static function add_save_icon($_header_buttons = '', $_id = 'p0'){
		// Define what reports get this new functionality
		if (!in_array($_id, array(
			'slim_p1_04','slim_p1_06','slim_p1_07','slim_p1_08','slim_p1_09','slim_p1_10','slim_p1_11','slim_p1_12','slim_p1_13','slim_p1_14',
			'slim_p2_03','slim_p2_04','slim_p2_05','slim_p2_06','slim_p2_07','slim_p2_09','slim_p2_10','slim_p2_13','slim_p2_14','slim_p2_15','slim_p2_16','slim_p2_17',
			'slim_p3_03','slim_p3_04','slim_p3_05','slim_p3_06','slim_p3_09','slim_p3_10','slim_p3_11',
			'slim_p4_02','slim_p4_03','slim_p4_04','slim_p4_05','slim_p4_06','slim_p4_07','slim_p4_08','slim_p4_10','slim_p4_11','slim_p4_12','slim_p4_13','slim_p4_14','slim_p4_15','slim_p4_16','slim_p4_17','slim_p4_18','slim_p4_19','slim_p4_20',
			'slim_p7_02',
			'slim_p8_01'
		))) return $_header_buttons;

		return '<a class="slimstat-filter-link button-export-to-xls slimstat-font-download" href="'.wp_slimstat_reports::fs_url("addon_e2e_id equals $_id").'" title="'.__('Export to CSV',self::$slug).'"></a> '.$_header_buttons;
	}

	public static function no_limits($_filters_normalized = array()){
		if (!empty($_POST['fs']['addon_e2e_id'])){
			$_filters_normalized['misc']['limit_results'] = 9999;
		}
		return $_filters_normalized;
	}

	/*
	 * Defines what happens when the user clicks the save button 
	 */
	public static function generate_xls(){
		if (!class_exists('wp_slimstat_db') || empty(wp_slimstat_db::$filters_normalized['columns']['addon_e2e_id'][1])) return true;

		$results = $columns = $headers = array();
		$total_for_percentage = 0;
		$first_column = '';
		$column_names = array_merge(wp_slimstat_db::$filter_names, array(
			'dt' => __('Date and Time',self::$slug),
			'plugin' => __('Plugin',self::$slug),
			'antialias' => __('Antialias',self::$slug),
			'outbound_domain' => __('Outbound Domain',self::$slug),
			'outbound_resource' => __('Outbound Link',self::$slug),
			'count' => __('Hits',self::$slug),
			'percentage' => __('Percentage',self::$slug),
			'ID' => __('ID',self::$slug),
			'user_login' => __('User Login',self::$slug),
			'user_email' => __('Email',self::$slug),
			'login_count' => __('Login Count',self::$slug),
			'pageviews' => __('Pageviews',self::$slug),
			'last_login' => __('Last Login',self::$slug),
			'time_on_site' => __('Time on Site',self::$slug)
		));

		switch(wp_slimstat_db::$filters_normalized['columns']['addon_e2e_id'][1]){
			case 'slim_p1_04':
				$results = wp_slimstat_db::get_recent('user', 't1.user <> "" AND t1.dt > '.(date_i18n('U')-300), '', '', '', false, false);
				$first_column = 'user';
				break;
			case 'slim_p1_06':
			case 'slim_p3_09':
				$results = wp_slimstat_db::get_recent('t1.searchterms');
				$first_column = 'searchterms';
				break;
			case 'slim_p1_07':
				$results = wp_slimstat_db::get_popular('t1.language', 't1.visit_id > 0 AND tb.type <> 1');
				$total_for_percentage = wp_slimstat_db::count_records('t1.visit_id > 0 AND tb.type <> 1');
				break;
			case 'slim_p1_08':
				$results = wp_slimstat_db::get_popular('SUBSTRING_INDEX(t1.resource, "?", 1)', '', '', '', 'resource');
				$total_for_percentage = wp_slimstat_db::count_records();

				break;
			case 'slim_p1_09':
			case 'slim_p2_13':
			case 'slim_p3_10':
				$results = wp_slimstat_db::get_recent('t1.country');
				$first_column = 'country';
				break;
			case 'slim_p1_10':
			case 'slim_p3_05':
				$results = wp_slimstat_db::get_popular('t1.domain');
				$total_for_percentage = wp_slimstat_db::count_records('t1.referer <> ""');
				break;
			case 'slim_p1_11':
				$results = wp_slimstat_db::get_popular('t1.user');
				$total_for_percentage = wp_slimstat_db::count_records('t1.user <> ""');
				break;
			case 'slim_p1_12':
			case 'slim_p3_03':
			case 'slim_p4_14':
				$results = wp_slimstat_db::get_popular('t1.searchterms');
				$total_for_percentage = wp_slimstat_db::count_records('t1.searchterms <> ""');
				break;
			case 'slim_p1_13':
			case 'slim_p2_10':
			case 'slim_p3_04':
				$results = wp_slimstat_db::get_popular('t1.country');
				$total_for_percentage = wp_slimstat_db::count_records();
				break;
			case 'slim_p1_14':
				$results = wp_slimstat_db::get_popular('tob.outbound_resource', 'tob.type = 1');
				$total_for_percentage = wp_slimstat_db::count_records('tob.outbound_resource <> "" AND tob.type = 1');
				break;
			case 'slim_p2_03':
				$results = wp_slimstat_db::get_popular('t1.language');
				$total_for_percentage = wp_slimstat_db::count_records();
				break;
			case 'slim_p2_04':
				$results = wp_slimstat_db::get_popular('tb.browser', '', ',tb.version');
				$total_for_percentage = wp_slimstat_db::count_records();
				break;
			case 'slim_p2_05':
				$results = wp_slimstat_db::get_popular('t1.ip');
				$total_for_percentage = wp_slimstat_db::count_records();
				break;
			case 'slim_p2_06':
				$results = wp_slimstat_db::get_popular('tb.platform');
				$total_for_percentage = wp_slimstat_db::count_records();
				break;
			case 'slim_p2_07':
				$results = wp_slimstat_db::get_popular('tss.resolution');
				$total_for_percentage = wp_slimstat_db::count_records('tss.resolution <> ""');
				break;
			case 'slim_p2_09':
                $wp_slim_plugins = array('flash', 'silverlight', 'acrobat', 'java', 'mediaplayer', 'director', 'real', 'quicktime');
                foreach($wp_slim_plugins as $i => $a_plugin){
                    $count_results = wp_slimstat_db::count_records("t1.plugins LIKE '%{$a_plugin}%'");
                    $results[] = array('plugin' => $a_plugin, 'count' => $count_results);
                }
                $total_for_percentage = wp_slimstat_db::count_records('t1.visit_id > 0 AND tb.type <> 1');
                break;
			case 'slim_p2_14':
				$results = wp_slimstat_db::get_recent('tss.resolution', '', 'tss.*');
				$first_column = 'resolution';
				break;
			case 'slim_p2_15':
				$results = wp_slimstat_db::get_recent('tb.platform', '', 'tb.*');
				$first_column = 'platform';
				break;
			case 'slim_p2_16':
				$results = wp_slimstat_db::get_recent('tb.browser', '', 'tb.*');
				$first_column = 'browser';
				break;
			case 'slim_p2_17':
				$results = wp_slimstat_db::get_recent('t1.language');
				$first_column = 'language';
				break;
			case 'slim_p3_11':
			case 'slim_p4_17':
				$results = wp_slimstat_db::get_popular('t1.resource', 't1.domain <> ""');
				$total_for_percentage = wp_slimstat_db::count_records('t1.domain <> ""');
				break;
			case 'slim_p3_06':
				$results = wp_slimstat_db::get_popular_complete('t1.domain', "t1.searchterms <> '' AND t1.domain <> '{$_SERVER['SERVER_NAME']}'");
				$total_for_percentage = wp_slimstat_db::count_records("t1.searchterms <> '' AND t1.domain <> '{$_SERVER['SERVER_NAME']}' AND t1.domain <> ''", 't1.id');
				break;
			case 'slim_p4_02':
				$results = wp_slimstat_db::get_recent('t1.resource', 'tci.content_type = "post"');
				$first_column = 'resource';
				break;
			case 'slim_p4_03':
				$results = wp_slimstat_db::get_recent('t1.resource', 'tci.content_type <> "404"', '', 'HAVING COUNT(visit_id) = 1');
				$first_column = 'resource';
				break;
			case 'slim_p4_04':
				$results = wp_slimstat_db::get_recent('t1.resource', '(t1.resource LIKE "%/feed%" OR t1.resource LIKE "%?feed=%" OR t1.resource LIKE "%&feed=%" OR tci.content_type LIKE "%feed%")');
				$first_column = 'resource';
				break;
			case 'slim_p4_05':
				$results = wp_slimstat_db::get_recent('t1.resource', '(t1.resource LIKE "[404]%" OR tci.content_type LIKE "%404%")');
				$first_column = 'resource';
				break;
			case 'slim_p4_06':
				$results = wp_slimstat_db::get_recent('t1.searchterms', '(t1.resource = "__l_s__" OR t1.resource = "" OR tci.content_type LIKE "%search%")');
				$first_column = 'searchterms';
				break;
			case 'slim_p4_07':
				$results = wp_slimstat_db::get_popular('tci.category', '(tci.content_type LIKE "%category%")', ',tci.category');
				$total_for_percentage = wp_slimstat_db::count_records('(tci.content_type LIKE "%category%")');
				break;
			case 'slim_p4_08':
				$results = wp_slimstat_db::get_recent('tob.outbound_resource', 'tob.type = 0');
				$first_column = 'outbound_resource';
				break;
			case 'slim_p4_10':
				$results = wp_slimstat_db::get_recent_outbound(-1);
				$first_column = 'resource';
				break;
			case 'slim_p4_10':
				$results = wp_slimstat_db::get_recent_outbound(-1);
				$first_column = 'resource';
				break;
			case 'slim_p4_11':
				$results = wp_slimstat_db::get_popular('t1.resource', 'tci.content_type = "post"');
				$total_for_percentage = wp_slimstat_db::count_records('tci.content_type = "post"');
				break;
			case 'slim_p4_12':
				$results = wp_slimstat_db::get_popular('t1.resource', '(t1.resource LIKE "%/feed%" OR t1.resource LIKE "%?feed=%" OR t1.resource LIKE "%&feed=%" OR tci.content_type LIKE "%feed%")');
				$total_for_percentage = wp_slimstat_db::count_records('(t1.resource LIKE "%/feed%" OR t1.resource LIKE "%?feed=%" OR t1.resource LIKE "%&feed=%" OR tci.content_type LIKE "%feed%")');
				break;
			case 'slim_p4_13':
				$results = wp_slimstat_db::get_popular('t1.searchterms', '(t1.resource = "__l_s__" OR t1.resource = "" OR tci.content_type LIKE "%search%")');
				$total_for_percentage = wp_slimstat_db::count_records('(t1.resource = "__l_s__" OR t1.resource = "" OR tci.content_type LIKE "%search%")');
				break;
			case 'slim_p4_15':
				$results = wp_slimstat_db::get_recent('t1.resource', '(tci.content_type = "category" OR tci.content_type = "tag")', 'tci.*');
				$first_column = 'resource';
				break;
			case 'slim_p4_16':
				$results = wp_slimstat_db::get_popular('t1.resource', '(t1.resource LIKE "[404]%" OR tci.content_type LIKE "%404%")');
				$total_for_percentage = wp_slimstat_db::count_records('(t1.resource LIKE "[404]%" OR tci.content_type LIKE "%404%")');
				break;
			case 'slim_p4_18':
				$results = wp_slimstat_db::get_popular('tci.author');
				$total_for_percentage = wp_slimstat_db::count_records('tci.author <> ""');
				break;
			case 'slim_p4_19':
				$results = wp_slimstat_db::get_popular('tci.category', '(tci.content_type LIKE "%tag%")', ',tci.category');
				$total_for_percentage = wp_slimstat_db::count_records('(tci.content_type LIKE "%tag%")');
				break;
			case 'slim_p4_20':
				$results = wp_slimstat_db::get_recent_outbound(1);
				$first_column = 'resource';
				break;
			case 'slim_p7_02':
				$results = wp_slimstat_db::get_recent('t1.id', '', 'tb.*,tci.*,tss.*');
				$first_column = 'ip';
				break;
			case 'slim_p8_01':
				$results = array();
				$users = wp_slimstat::$wpdb->get_results("
					SELECT tu.ID, tu.user_login, tu.user_email, tum.meta_value login_count, COUNT(*) pageviews, MAX(t1.dt) last_login
					FROM {$GLOBALS['wpdb']->dbname}.{$GLOBALS['wpdb']->users} tu
						INNER JOIN {$GLOBALS['wpdb']->dbname}.{$GLOBALS['wpdb']->usermeta} tum2 ON (tum2.user_id = tu.ID AND tum2.meta_key =  '{$GLOBALS['wpdb']->prefix}capabilities')
						LEFT JOIN {$GLOBALS['wpdb']->dbname}.{$GLOBALS['wpdb']->usermeta} tum ON (tum.user_id = tu.ID AND tum.meta_key =  'slimstat_login_counter')
						LEFT JOIN (".wp_slimstat_db::$sql_filters['from']['all_tables'].") ON tu.user_login = t1.user
					WHERE t1.dt IS NOT NULL ".wp_slimstat_db::$sql_filters['where'].' '.wp_slimstat_db::$sql_filters['where_time_range'].'
					GROUP BY tu.user_login
					ORDER BY tu.user_login ASC', ARRAY_A);

				if (empty(wp_slimstat_db::$sql_filters['where'])){
					$no_pageviews_users = wp_slimstat::$wpdb->get_results("
						SELECT tu.ID, tu.user_login, tu.user_email, tum.meta_value login_count, 0 pageviews, 0 last_login
						FROM {$GLOBALS['wpdb']->dbname}.{$GLOBALS['wpdb']->users} tu
							INNER JOIN {$GLOBALS['wpdb']->dbname}.{$GLOBALS['wpdb']->usermeta} tum2 ON (tum2.user_id = tu.ID AND tum2.meta_key =  '{$GLOBALS['wpdb']->prefix}capabilities')
							LEFT JOIN {$GLOBALS['wpdb']->dbname}.{$GLOBALS['wpdb']->usermeta} tum ON (tum.user_id = tu.ID AND tum.meta_key =  'slimstat_login_counter')
							LEFT JOIN {$GLOBALS['wpdb']->prefix}slim_stats t1 ON tu.user_login = t1.user
						WHERE t1.dt IS NULL
						GROUP BY tu.user_login
						ORDER BY tu.user_login ASC", ARRAY_A);
					if (!empty($no_pageviews_users)) $users = array_merge($users, $no_pageviews_users);
				}
				
				$all_visit_durations = wp_slimstat::$wpdb->get_results("
					SELECT MAX(t1.dt)-MIN(t1.dt) visit_duration, t1.user, t1.visit_id
					FROM {$GLOBALS['wpdb']->prefix}slim_stats t1
					WHERE t1.user <> '' AND t1.visit_id <> 0 ".wp_slimstat_db::$sql_filters['where'].' '.wp_slimstat_db::$sql_filters['where_time_range']."
					GROUP BY t1.user, t1.visit_id
					HAVING visit_duration > 0", ARRAY_A);

				foreach ($users as $a_user){
					$total_time_on_site = 0;
					if(intval($a_user['last_login'])!=0){
						foreach($all_visit_durations as $idx => $a_duration){
							if ($a_duration['user'] == $a_user['user_login']){
								$total_time_on_site += $a_duration['visit_duration'];
								unset($all_visit_durations[$idx]);
							}
						}
					}
					$login_count = intval(get_user_meta($a_user['ID'], 'slimstat_login_counter', true));
					$results[] = array_merge($a_user, array('time_on_site' => $total_time_on_site, 'login_count' => $login_count));
				}
				$first_column = 'user_login';
				break;
			default:
				break;
		}

		if (!empty($results)){
			$columns = array_diff(array_keys($results[0]), array('id', 'browser_id', 'screenres_id', 'content_info_id', 'content_id', 'maxid', $first_column));
			if (!empty($first_column)) array_unshift($columns, $first_column);
		}
		if (!empty($total_for_percentage)) $columns[] = 'percentage';

		header('Content-Type: text/csv; charset=utf-8');
		header('Content-Disposition: attachment; filename='.(!empty(wp_slimstat_reports::$all_reports_titles[wp_slimstat_db::$filters_normalized['columns']['addon_e2e_id'][1]])?sanitize_title(wp_slimstat_reports::$all_reports_titles[wp_slimstat_db::$filters_normalized['columns']['addon_e2e_id'][1]]):'data').'.csv');

		$output = fopen('php://output', 'w');
		foreach ($columns as $a_column_key){
			$headers[] = $column_names[$a_column_key];
		}

		fputcsv($output, $headers);
		foreach($results as $a_result){
			$a_row = array();
			foreach ($columns as $a_column_key){
				switch($a_column_key){
					case 'dt':
					case 'last_login':
						$a_row[$a_column_key] = date_i18n(wp_slimstat_db::$formats['date_time_format'], $a_result[$a_column_key], true);
						break;
					case 'ip':
					case 'other_ip':
						$a_row[$a_column_key] = long2ip($a_result[$a_column_key]);
						break;
					case 'percentage':
						$a_row['percentage'] = ($total_for_percentage > 0)?number_format(sprintf("%01.4f", (100*$a_result['count']/$total_for_percentage)), 4):0;
						break;
					case 'time_on_site':
						$a_row['time_on_site'] = date_i18n('H:i:s', $a_result['time_on_site']);
						break;
					case 'pageviews':
						$a_row['pageviews'] = (!empty($a_result['pageviews']) && $a_result['last_login']!=0)?$a_result['pageviews']:0;
						break;
					default:
						$a_row[$a_column_key] = $a_result[$a_column_key];
						break;
				}
			}
			fputcsv($output, $a_row);
		}
		fclose($output);
		die(0);
	}
}
// end of class declaration

// Bootstrap
if (function_exists('add_action')){
	add_action('plugins_loaded', array('wp_slimstat_export_to_excel', 'init'), 15);
}