<?php


class wp_slimstat_ip2location_details{
	public static $slug = 'wp-slimstat-ip2location-details';

	/**
	 * Initializes the environment
	 */
	public static function init(){
		if (!class_exists('wp_slimstat')) return true;

		// Update checker
		add_action('wp_loaded', array(__CLASS__, 'update_checker'));

		// Define the page that will return the content for the iframe
		if (!empty($_GET['action']) && $_GET['action'] == 'kws_ip2location_iframe_content'){
			add_action('wp_ajax_kws_ip2location_iframe_content', array(__CLASS__, 'ip2location_iframe_content'));
		}

		// Add a new option to the Advanced Settings tab
		add_filter('slimstat_options_on_page', array(__CLASS__, 'add_license_key_option'));

		// Modify the value associated to the ip_lookup_url option
		add_filter('slimstat_init_options', array(__CLASS__, 'force_local_geolocation_url'));

		// Load localization files
		load_plugin_textdomain(self::$slug, false, self::$slug . '/lang/');
	}
	
	/**
	 * Checks for updates
	 */
	public static function update_checker(){
		if (!empty(wp_slimstat::$options['addon_licenses'][self::$slug])){
			@include_once(plugin_dir_path( __FILE__ ).'update-checker.php');
			$MyUpdateChecker = new PluginUpdateChecker('http://slimstat.getused.to.it/update-checker/?slug='.self::$slug.'&key='.urlencode(wp_slimstat::$options['addon_licenses'][self::$slug]), __FILE__, self::$slug);
		}
	}

	/**
	 * Processes the geolocation request
	 */
	public static function ip2location_iframe_content(){
		if (empty($_GET['ip']) || long2ip(ip2long($_GET['ip'])) !=  $_GET['ip']){
			_e('An error occurred during the request. Have you <a href="http://www.ip2location.com/" target="_blank">purchased</a> your IP2Location License Key yet?', self::$slug);
			exit(-1);
		}
		
		if (empty(wp_slimstat::$options['addon_ip2location_license_key'])){
			$remote_api_url = 'http://slimstat.ip2location.com/?ip='.$_GET['ip'];
		}
		else{
			$remote_api_url = 'http://slimstat.ip2location.com/?ip='.$_GET['ip'].'&key='.wp_slimstat::$options['addon_ip2location_license_key'];
		}
		
		$response = wp_remote_get($remote_api_url);
		if(is_wp_error($response) || $response['response']['code'] != 200){
			$error_message = is_wp_error($response)?$response->get_error_message():$response['response']['code'].' '. $response['response']['message'];
			echo '<p>'.__('There was an error retrieving the information from the server. Please try again later. Error Message:',self::$slug).' '.$error_message.'</p></div>';
			exit(-2);
		}

		$data = @json_decode($response['body'], true);

		// Import and initialize the API to interact with the database
		@include_once(WP_PLUGIN_DIR."/wp-slimstat/admin/view/wp-slimstat-db.php");
		wp_slimstat_db::init('ip equals '.$_GET['ip']);
		$ip2long = sprintf("%u", ip2long($_GET['ip']));

		?>
		<html>
		<head>
			<title>Geolocation Information</title>
			<style type="text/css">
				body{ font-family:sans-serif; margin:0 }
				table{ border-spacing:0; font-size:12px; float:left; line-height:1.5em; width:49%; }
				table td{ border-top:1px solid #808080; padding:5px; }
				table td.powered-by{ font-size:.8em; text-align:right; }
				table td.powered-by a{ color:#fff; }
				table td.name{ width:35%; }
				table thead td{ background:#808080; color:#fff; }
			</style>
		</head>
		<body>
			<table style="margin-right:2%">
			<thead>
				<tr>
					<td><?php _e('IP Geolocation Information', self::$slug) ?></td>
					<td class="powered-by"><?php _e('Powered by <a target="_blank" href="http://www.ip2location.com/">IP2Location.com</a>', self::$slug) ?></td>
				</tr>
			</thead>
			<tbody>
				<tr class="alt">
					<td class="name"><?php _e('IP Address', self::$slug) ?></td>
					<td><?php echo $_GET['ip'] ?></td>
				</tr>
				<tr>
					<td class="name"><?php _e('Country Code', self::$slug) ?></td>
					<td><?php echo $data['country_code'] ?></td>
				</tr>
				<tr>
					<td class="name"><?php _e('City &amp; Region', self::$slug) ?></td>
					<td><?php echo ucfirst(strtolower($data['city_name'])).', '.ucfirst(strtolower($data['region_name'])) ?></td>
				</tr>
				<tr>
					<td class="name"><?php _e('Latitude &amp; Longitude', self::$slug) ?></td>
					<td><?php echo $data['latitude'].', '.$data['longitude'] ?></td>
				</tr>
				<tr class="alt">
					<td class="name"><?php _e('ZIP Code', self::$slug) ?></td>
					<td><?php echo $data['zip_code'] ?></td>
				</tr>
				<tr>
					<td class="name"><?php _e('Area Code', self::$slug) ?></td>
					<td><?php echo $data['area_code'] ?></td>
				</tr>
				<tr>
					<td class="name"><?php _e('Time Zone', self::$slug) ?></td>
					<td><?php echo $data['time_zone'] ?></td>
				</tr>
				<tr>
					<td class="name"><?php _e('International Area Code', self::$slug) ?></td>
					<td><?php echo $data['idd_code'] ?></td>
				</tr>
			</tbody>
			</table>
			
			<table>
			<thead>
				<tr>
					<td colspan="2"><?php _e('Activity', self::$slug) ?></td>
				</tr>
			</thead>
			<tbody>
				<tr>
					<td class="name"><?php _e('Oldest Pageview', self::$slug) ?></td>
					<td><?php $dt = wp_slimstat_db::get_oldest_visit(); echo ($dt == null)?__('No visits','wp-slimstat'):date_i18n(get_option('date_format'), $dt); ?></td>
				</tr>
				<tr>
					<td class="name"><?php _e('Total Visits', self::$slug) ?></td>
					<td><?php echo number_format(wp_slimstat_db::count_records('1=1', '*', true, false), 0, wp_slimstat_db::$formats['decimal'], wp_slimstat_db::$formats['thousand']) ?></td>
				</tr>
				<tr>
					<td class="name"><?php _e('Pageviews This Month', self::$slug) ?></td>
					<td><?php echo number_format(wp_slimstat_db::count_records('1=1', '*', true, true), 0, wp_slimstat_db::$formats['decimal'], wp_slimstat_db::$formats['thousand']) ?></td>
				</tr>
				<tr>
					<td class="name"><?php _e('Browsers', self::$slug) ?></td>
					<td><?php
						$results_flat = array();
						$results = $GLOBALS['wpdb']->get_results("
							SELECT DISTINCT tb.browser as a_value
							FROM {$GLOBALS['wpdb']->base_prefix}slim_browsers tb INNER JOIN {$GLOBALS['wpdb']->prefix}slim_stats t1 ON tb.browser_id = t1.browser_id
							WHERE t1.ip = '$ip2long'", ARRAY_A);
							
						foreach($results as $a_result) $results_flat[] = $a_result['a_value'];
						echo implode(', ', $results_flat);
					?></td>
				</tr>
				<tr>
					<td class="name"><?php _e('Screen Resolutions', self::$slug) ?></td>
					<td><?php
						$results_flat = array();
						$results = $GLOBALS['wpdb']->get_results("
							SELECT DISTINCT tss.resolution as a_value
							FROM {$GLOBALS['wpdb']->base_prefix}slim_screenres tss INNER JOIN {$GLOBALS['wpdb']->prefix}slim_stats t1 ON tss.screenres_id = t1.screenres_id
							WHERE t1.ip = '$ip2long'", ARRAY_A);
							
						foreach($results as $a_result) $results_flat[] = $a_result['a_value'];
						echo implode(', ', $results_flat);
					?></td>
				</tr>
				<tr>
					<td class="name"><?php _e('Total Time on Site', self::$slug) ?></td>
					<td><?php
						$total_time = 0;
						
						$results = $GLOBALS['wpdb']->get_results("
							SELECT MAX(t1.dt)-MIN(t1.dt) visit_duration, t1.visit_id
							FROM {$GLOBALS['wpdb']->prefix}slim_stats t1
							WHERE t1.ip = '$ip2long' AND t1.visit_id <> 0
							GROUP BY t1.visit_id
							HAVING visit_duration > 0", ARRAY_A);

							foreach($results as $a_result) $total_time += $a_result['visit_duration'];
						echo date_i18n('H:i:s', $total_time);
					?></td>
				</tr>
				<tr>
					<td class="name"><?php _e('Search Terms', self::$slug) ?></td>
					<td><?php
						$results_flat = array();
						$results = $GLOBALS['wpdb']->get_results("
							SELECT DISTINCT t1.searchterms as a_value
							FROM {$GLOBALS['wpdb']->prefix}slim_stats t1
							WHERE t1.ip = '$ip2long'", ARRAY_A);
							
						foreach($results as $a_result) $results_flat[] = $a_result['a_value'];
						echo implode(', ', $results_flat);
					?></td>
				</tr>
				<tr>
					<td class="name"><?php _e('Referring Domains', self::$slug) ?></td>
					<td><?php
						$results_flat = array();
						$results = $GLOBALS['wpdb']->get_results("
							SELECT DISTINCT t1.domain as a_value
							FROM {$GLOBALS['wpdb']->prefix}slim_stats t1
							WHERE t1.domain <> '' AND t1.ip = '$ip2long'", ARRAY_A);
							
						foreach($results as $a_result) $results_flat[] = $a_result['a_value'];
						echo implode(', ', $results_flat);
					?></td>
				</tr>
			</tbody>
		</table>
		</body>
		</html><?php
		die();
	}

	/**
	 * Adds a new option to the Advanced Settings tab
	 */
	public static function add_license_key_option($_options = array()){
		$_options['addon_ip2location_license_key'] = array('description' => __('IP2Location License Key',self::$slug), 'type' => 'text', 'long_description' => __('You can perfom up to 50 queries per day. If you need more queries, please <a href="http://keywordswarm.com/" target="_blank">visit the official page</a> to purchase a new key for the GeoIP City/ISP/Organization service.',self::$slug));
		return $_options;
	}

	/**
	 * Modifies the value associated to the ip_lookup_url option
	 */
	public static function force_local_geolocation_url($_options){
		$_options['ip_lookup_service'] = admin_url( 'admin-ajax.php' ).'?action=kws_ip2location_iframe_content&ip=';
		return $_options;
	}
}
// end of class declaration

// Bootstrap
if (function_exists('add_action')){
	add_action('plugins_loaded', array('wp_slimstat_ip2location_details', 'init'), 5); // WP SlimStat's priority is set to 10 - We need to setup all the filters BEFORE SlimStat is initialized
}