<?php


class wp_slimstat_label_ip_addresses{
	public static $slug = 'wp-slimstat-label-ip-addresses';

	/**
	 * Initializes the environment
	 */
	public static function init(){
		if (!class_exists('wp_slimstat')) return true;

		// Update checker
		if (!empty(wp_slimstat::$options['addon_licenses'][self::$slug])){
			add_action('wp_loaded', array(__CLASS__, 'update_checker'));
		}

		// Option to associate labels to IP Addresses
		add_filter('slimstat_init_options', array(__CLASS__, 'init_options'));

		// Adds a new field to the Settings page
		add_filter('slimstat_options_on_page', array(__CLASS__, 'add_textarea_to_options_page'));
		
		// Updates the $stat array if a match is found
		add_filter('slimstat_filter_pageview_stat', array(__CLASS__, 'lookup_ip_address'), 10, 3);

		// Load localization files
		load_plugin_textdomain(self::$slug, false, self::$slug . '/lang/');
	}

	/**
	 * Checks for updates
	 */
	public static function update_checker(){
		@include_once(plugin_dir_path( __FILE__ ).'update-checker.php');
		$MyUpdateChecker = new PluginUpdateChecker('http://slimstat.getused.to.it/update-checker/?slug='.self::$slug.'&key='.urlencode(wp_slimstat::$options['addon_licenses'][self::$slug]), __FILE__, self::$slug);
	}

	public static function lookup_ip_address($_stat = array(), $_browser = array(), $_content_info = array()){
		foreach(wp_slimstat::string_to_array(wp_slimstat::$options['addon_label_ip_addresses_pairs']) as $a_pair){
			list($ip_address, $label) = @explode('=', trim($a_pair));
			
			// Support for CIDR Notation 
			if (strpos($ip_address, '/') !== false){
				list($ip_address, $mask) = @explode('/', trim($ip_address));
			}
			if (empty($mask) || !is_numeric($mask)) $mask = 32;
			$long_ip_address = ip2long($ip_address);
			$long_mask = bindec( str_pad('', $mask, '1') . str_pad('', 32-$mask, '0') );
			if (($_stat['ip'] & $long_mask) == ($long_ip_address & $long_mask)){
				$_stat['user'] = $label;
			}
		}
		return $_stat;
	}

	/**
	 * Adds a new option to the Settings > Add-ons tab
	 */
	public static function add_textarea_to_options_page($_options = array()){
		$_options['addon_label_ip_addresses_pairs'] = array('description' => __('IP to Labels',self::$slug), 'type' => 'textarea', 'long_description' => __('Specify the pairs IP=LABEL, separate by commas. You can use the CIDR notation to associate the same label to groups of multiple IP addresses. If a given IP matches more than one label, the last one listed will be used.',self::$slug));
		return $_options;
	}

	/**
	 * Modifies the value associated to the ip_lookup_url option
	 */
	public static function init_options($_options){
		if (!isset($_options['addon_label_ip_addresses_pairs'])) $_options['addon_label_ip_addresses_pairs'] = '';
		return $_options;
	}
}
// end of class declaration

// Bootstrap
if (function_exists('add_action')){
	add_action('plugins_loaded', array('wp_slimstat_label_ip_addresses', 'init'), 8);
}