<?php


class wp_slimstat_user_overview{
	public static $slug = 'wp-slimstat-user-overview';
	public static $report_id = 'slim_p8_01';

	/**
	 * Initializes the environment
	 */
	public static function init(){
		if (!class_exists('wp_slimstat')) return true;

		// Update checker
		add_action('wp_loaded', array(__CLASS__, 'update_checker'));

		// Attach the report to the Custom Reports tab
		add_filter('wp_slimstat_custom_report', array(__CLASS__, 'user_information'));

		// Count login attempts
		add_action('wp_login', array(__CLASS__, 'set_last_login_time'), 1, 2);

		// Localization files
		load_plugin_textdomain(self::$slug, false, self::$slug . '/lang/');

		// Add a new option to the Add-ons Settings tab
		add_filter('slimstat_options_on_page', array(__CLASS__, 'add_user_overview_options'));

		// Add custom report to the WordPress Dashboard
		if (function_exists('is_network_admin') && !is_network_admin()){
			add_action('wp_dashboard_setup', array(__CLASS__, 'add_dashboard_widget'));
		}

		// Add the ajax action to handle dynamic report updates
		if (!empty($_POST['action']) && $_POST['action'] == 'slimstat_load_report'){
			add_action('wp_ajax_slimstat_load_report', array(__CLASS__, 'user_information'));
		}

		// Add report title
		add_filter('slimstat_report_titles', array(__CLASS__, 'report_title'));
	}

	/**
	 * Checks for updates
	 */
	public static function update_checker(){
		if (!empty(wp_slimstat::$options['addon_licenses'][self::$slug])){
			@include_once(plugin_dir_path( __FILE__ ).'update-checker.php');
			$MyUpdateChecker = new PluginUpdateChecker('http://slimstat.getused.to.it/update-checker/?slug='.self::$slug.'&key='.urlencode(wp_slimstat::$options['addon_licenses'][self::$slug]), __FILE__, self::$slug);
		}
	}

	/**
	 * Attaches the custom report to the dashboard
	 */
	public static function add_dashboard_widget(){
		if (!empty(wp_slimstat::$options['addon_user_overview_enable_dashboard']) && wp_slimstat::$options['addon_user_overview_enable_dashboard'] == 'no'){
			return true;
		}

		// If this user is whitelisted, we use the minimum capability
		if (strpos(wp_slimstat::$options['can_view'], $GLOBALS['current_user']->user_login) === false){
			$minimum_capability = wp_slimstat::$options['capability_can_view'];
		}
		else{
			$minimum_capability = 'read';
		}

		if (!current_user_can($minimum_capability)) return;

		wp_add_dashboard_widget(self::$report_id, __('User Overview',self::$slug), array(__CLASS__, 'user_information'));
	}
	// end add_dashboard_widgets

	public static function report_title($_titles = array()){
		$_titles[self::$report_id] = __('User Overview',self::$slug);
		return $_titles;
	}

	public static function user_information($_report_id = 'p0'){
		if (!empty($_POST['report_id']) && $_POST['report_id'] != self::$report_id){
			return true;
		}

		// Don't run the code, if the report is hidden
		$screen = get_current_screen();
		$user = wp_get_current_user();
		if (is_object($screen)){
			$page_location = $screen->id;
			$hidden_reports = get_user_option("metaboxhidden_$page_location", $user->ID);
			if (is_array($hidden_reports) && in_array(self::$report_id, $hidden_reports)) return true;
		}

		$is_dashboard = false;
		$orderby_multisort = $orderby_field = 'pageviews';
		if (!class_exists('wp_slimstat_reports')){
			include_once(plugin_dir_path( dirname(__FILE__ )).'wp-slimstat/admin/view/wp-slimstat-reports.php');
			wp_slimstat_reports::init();
			$is_dashboard = true;
		}
		else{
			wp_slimstat_db::$filters_normalized['misc']['limit_results'] = wp_slimstat::$options['number_results_raw_data'];
			if (!empty(wp_slimstat_db::$filters_normalized['columns']['addon_uo_orderby'][1])){
				if (in_array(wp_slimstat_db::$filters_normalized['columns']['addon_uo_orderby'][1], array('user_login','user_email', 'pageviews'))){
					$orderby_multisort = $orderby_field = wp_slimstat_db::$filters_normalized['columns']['addon_uo_orderby'][1];
				}
				else if (in_array(wp_slimstat_db::$filters_normalized['columns']['addon_uo_orderby'][1], array('login_count'))){ // numeric values
					$orderby_multisort = wp_slimstat_db::$filters_normalized['columns']['addon_uo_orderby'][1];
					$orderby_field = "$orderby_multisort + 0";
				}
			}
			$multisort = (wp_slimstat_db::$filters_normalized['misc']['direction'] == 'desc')?SORT_DESC:SORT_ASC;
		}

		// Set the filters
		$global_filters = trim(wp_slimstat_db::$sql_filters['where'].' '.wp_slimstat_db::$sql_filters['where_time_range']);
		$restrict_current_blog = (isset(wp_slimstat::$options['addon_user_overview_global_users']) && wp_slimstat::$options['addon_user_overview_global_users'] == 'yes')?"INNER JOIN {$GLOBALS['wpdb']->dbname}.{$GLOBALS['wpdb']->usermeta} tum2 ON (tum2.user_id = tu.ID AND tum2.meta_key =  '{$GLOBALS['wpdb']->prefix}capabilities')":'';

		// Get the data
		$all_results = wp_slimstat::$wpdb->get_results("
			SELECT tu.ID, tu.user_login, tu.user_email, tum.meta_value login_count, COUNT(*) pageviews, MAX(t1.dt) maxdt, MIN(t1.dt) mindt
			FROM {$GLOBALS['wpdb']->dbname}.{$GLOBALS['wpdb']->users} tu
				$restrict_current_blog
				INNER JOIN (".wp_slimstat_db::$sql_filters['from']['all_tables'].") ON tu.user_login = t1.user
				LEFT JOIN {$GLOBALS['wpdb']->dbname}.{$GLOBALS['wpdb']->usermeta} tum ON (tum.user_id = tu.ID AND tum.meta_key =  'slimstat_login_counter')
			WHERE 1=1 $global_filters
			GROUP BY tu.user_login
			ORDER BY $orderby_field ".wp_slimstat_db::$filters_normalized['misc']['direction'], ARRAY_A);

		$distinct_logins = array();
		foreach ($all_results as $a_result){
			$distinct_logins[] = $a_result['user_login'];
		}
		$distinct_logins = array_unique($distinct_logins);

		if (empty(wp_slimstat_db::$sql_filters['where'])){
			if (!empty($distinct_logins)){
				$no_pageviews_results = wp_slimstat::$wpdb->get_results(wp_slimstat::$wpdb->prepare("
					SELECT tu.ID, tu.user_login, tu.user_email, tum.meta_value login_count, 0 pageviews, 0 maxdt, 0 mindt
					FROM {$GLOBALS['wpdb']->dbname}.{$GLOBALS['wpdb']->users} tu
						$restrict_current_blog
						LEFT JOIN {$GLOBALS['wpdb']->dbname}.{$GLOBALS['wpdb']->usermeta} tum ON (tum.user_id = tu.ID AND tum.meta_key = 'slimstat_login_counter')
					WHERE tu.user_login NOT IN (".substr(str_repeat('%s,', count($distinct_logins)), 0, -1).")
					GROUP BY tu.user_login
					ORDER BY $orderby_field ".wp_slimstat_db::$filters_normalized['misc']['direction'], $distinct_logins), ARRAY_A);
			}
			else{
				$no_pageviews_results = wp_slimstat::$wpdb->get_results("
					SELECT tu.ID, tu.user_login, tu.user_email, tum.meta_value login_count, 0 pageviews, 0 maxdt, 0 mindt
					FROM {$GLOBALS['wpdb']->dbname}.{$GLOBALS['wpdb']->users} tu
						$restrict_current_blog
						LEFT JOIN {$GLOBALS['wpdb']->dbname}.{$GLOBALS['wpdb']->usermeta} tum ON (tum.user_id = tu.ID AND tum.meta_key = 'slimstat_login_counter')
					GROUP BY tu.user_login
					ORDER BY $orderby_field ".wp_slimstat_db::$filters_normalized['misc']['direction'], ARRAY_A);
			}
			if (!empty($no_pageviews_results)){
				$all_results = array_merge($all_results, $no_pageviews_results);

				// Sort merged arrays
				foreach($all_results as $key => $row){
					$temp_sort[$key] = $row[$orderby_multisort];

				}

				array_multisort($temp_sort, (wp_slimstat_db::$filters_normalized['misc']['direction'] == 'desc')?SORT_DESC:SORT_ASC, $all_results);
				
			}
		}

		$all_visit_durations = wp_slimstat::$wpdb->get_results("
			SELECT MAX(t1.dt)-MIN(t1.dt) visit_duration, t1.user, t1.visit_id
			FROM {$GLOBALS['wpdb']->prefix}slim_stats t1
			WHERE t1.user <> '' AND t1.visit_id > 0 ".wp_slimstat_db::$sql_filters['where'].' '.wp_slimstat_db::$sql_filters['where_time_range']."
			GROUP BY t1.visit_id
			HAVING visit_duration > 0", ARRAY_A);

		$results = array_slice($all_results, wp_slimstat_db::$filters_normalized['misc']['start_from'], wp_slimstat_db::$filters_normalized['misc']['limit_results']);
		$count_page_results = count($results);

		if (!empty($_GET['page'])){
			wp_slimstat_reports::report_header(self::$report_id, 'tall');
		} 
		wp_slimstat_reports::report_pagination(self::$report_id, $count_page_results, count($all_results)); 
		
		$opposite_direction = (wp_slimstat_db::$filters_normalized['misc']['direction'] == 'asc')?'desc':'asc';
		$display_ordering_tools = !empty($_GET['page']) || (!empty($_POST['current_tab']) && $_POST['current_tab'] == 7);
		?>

		<table class="wp-list-table widefat fixed users" cellspacing="0">
			<thead>
			<tr>
				<th scope="col" id="username" class="manage-column column-username sortable <?php if ($display_ordering_tools) echo wp_slimstat_db::$filters_normalized['misc']['direction'] ?>">
					<?php if ($display_ordering_tools): ?><a class="slimstat-filter-link" href="<?php echo wp_slimstat_reports::fs_url('direction equals '.$opposite_direction.'&&&addon_uo_orderby equals user_login') ?>"><?php endif ?><span><?php _e('Username',self::$slug) ?></span><?php if ($display_ordering_tools): ?><span class="sorting-indicator"></span></a><?php endif ?>
				</th>
				<?php if (!$is_dashboard): ?>
				<th scope="col" class="manage-column">Full Name</th>
				<th scope="col" class="manage-column column-email sortable <?php if ($display_ordering_tools) echo wp_slimstat_db::$filters_normalized['misc']['direction'] ?>">
					<?php if ($display_ordering_tools): ?><a class="slimstat-filter-link" href="<?php echo wp_slimstat_reports::fs_url('direction equals '.$opposite_direction.'&&&addon_uo_orderby equals user_email') ?>"><?php endif ?><span><?php _e('Email',self::$slug) ?></span><?php if ($display_ordering_tools): ?><span class="sorting-indicator"></span></a><?php endif ?>
				</th>
				<th scope="col" class="manage-column">Last Login</th>
				<?php endif ?>
				<th scope="col" class="manage-column sortable <?php if ($display_ordering_tools) echo wp_slimstat_db::$filters_normalized['misc']['direction'] ?>">
					<?php if ($display_ordering_tools): ?><a class="slimstat-filter-link" href="<?php echo wp_slimstat_reports::fs_url('direction equals '.$opposite_direction.'&&&addon_uo_orderby equals pageviews') ?>"><?php endif ?><span><?php _e('Pageviews',self::$slug) ?></span><?php if ($display_ordering_tools): ?><span class="sorting-indicator"></span></a><?php endif ?>
				</th>
				<th scope="col" class="manage-column sortable <?php if ($display_ordering_tools) echo wp_slimstat_db::$filters_normalized['misc']['direction'] ?>">
					<?php if ($display_ordering_tools): ?><a class="slimstat-filter-link" href="<?php echo wp_slimstat_reports::fs_url('direction equals '.$opposite_direction.'&&&addon_uo_orderby equals login_count') ?>"><?php endif ?><span><?php _e('Login Count',self::$slug) ?></span><?php if ($display_ordering_tools): ?><span class="sorting-indicator "></span></a><?php endif ?>
				</th>
				<th scope="col" class="manage-column">Total Time On Site</th>
			</tr>
			</thead>

			<tbody id="the-list" data-wp-lists="list:user">
				<?php
					foreach ($results as $i => $a_user): 
						$total_time_on_site = 0;
						if(intval($a_user['maxdt'])!=0){
							foreach($all_visit_durations as $idx => $a_duration){
								if ($a_duration['user'] == $a_user['user_login']){
									$total_time_on_site += $a_duration['visit_duration'];
									unset($all_visit_durations[$idx]);
								}
							}
						}
					?>
				<tr id="user-<?php echo $a_user['user_login'] ?>"<?php echo ($i%2!=0)?' class="alternate"':'' ?>>
					<td class="username column-username"><strong><?php echo "<a class='slimstat-filter-link' title='".htmlentities(sprintf(__('Filter results where user equals %s',self::$slug), $a_user['user_login']), ENT_QUOTES, 'UTF-8')."' href='".wp_slimstat_reports::fs_url('user equals '.$a_user['user_login'].'&&&direction equals desc', str_replace('slim-view-7', 'slim-view-2', wp_slimstat_reports::$view_url))."'>{$a_user['user_login']}</a>"; ?></td>
					<?php if (!$is_dashboard): ?>
					<td class="name column-name"><?php $full_name = trim(get_the_author_meta('first_name', $a_user['ID']).' '.get_the_author_meta('last_name', $a_user['ID'])); echo empty($full_name)?$a_user['user_login']:$full_name; ?></td>
					<td class="name column-name"><?php echo $a_user['user_email'] ?></td>
					<td class="name column-name"><?php echo self::get_last_login_time($a_user['user_login']) ?></td>
					<?php endif ?>
					<td class="name column-name"><?php echo (!empty($a_user['pageviews']) && $a_user['maxdt']!=0)?$a_user['pageviews']:0 ?></td>
					<td class="name column-name"><?php echo intval(get_user_meta($a_user['ID'], 'slimstat_login_counter', true)); ?></td>
					<td class="name column-name"><?php echo date_i18n('H:i:s', $total_time_on_site); ?></td>
				</tr>
				<?php endforeach ?>
			</tbody>
		</table><?php 
		if (!empty($_GET['page'])){
			wp_slimstat_reports::report_footer();
		}
	}
	
	public static function set_last_login_time($_user_login, $_user = 0){
		// We do not rely on $_user being properly initialized, as this appears not to be the case sometimes
		$user_object = get_user_by('login', $_user_login);
		update_usermeta($user_object->ID, 'slimstat_last_login', date_i18n('U'));

		$login_counter = self::get_login_counter($_user_login);
		update_usermeta($user_object->ID, 'slimstat_login_counter', $login_counter+1);
	}
	
	public static function get_last_login_time($_user_login){
		$user = get_user_by('login', $_user_login);
		$last_login = get_user_meta($user->ID, 'slimstat_last_login', true);
		return empty($last_login)?__('Never',self::$slug):date_i18n(wp_slimstat_db::$formats['date_time_format'], $last_login, true);
	}
	
	public static function get_login_counter($_user_login){
		$user = get_user_by('login', $_user_login);
		return intval(get_user_meta($user->ID, 'slimstat_login_counter', true));
	}
	
	/**
	 * Adds a new option to the Advanced Settings tab
	 */
	public static function add_user_overview_options($_options = array()){
		$_options['addon_user_overview_header'] = array('description' => __('User Overview',self::$slug), 'type' => 'section_header');
		$_options['addon_user_overview_global_users'] = array('description' => __('Site Mode',self::$slug), 'type' => 'yesno', 'long_description' => __('Restrict the report to users associated to the current site in your network.',self::$slug));
		$_options['addon_user_overview_enable_dashboard'] = array('description' => __('Add to Dashboard',self::$slug), 'type' => 'yesno', 'long_description' => __('Add the User Overview report to your WordPress Dashboard.',self::$slug));
		return $_options;
	}
}
// end of class declaration

// Bootstrap
if (function_exists('add_action')){
	add_action('plugins_loaded', array('wp_slimstat_user_overview', 'init'), 15);
}